#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['bws_scheduler']

package_data = \
{'': ['*']}

setup(name='bws_scheduler',
      version='0.0.1a0',
      description='BWS Scheduler',
      author='Jose Angel Delgado',
      author_email='esojangel@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
     )
